# TPROC - Run image processing tasks on both TRES amps, combine, and flatten
# September 19, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=no	{prompt="Make preamp correction (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	flatten=yes	{prompt="Run tflat on each image (yes or no)"}
bool	cosmic=yes	{prompt="Remove cosmic rays from a list of images (yes or no)"}
bool	extract=yes	{prompt="Extract spectra using textract (yes or no)"}
bool	disperse=yes	{prompt="Add wavelength dispersion function to spectrum (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
string	sumspec="no"	{prompt="Process summed 2-D file after tcosmic (yes no mean only)"}
string	aproot="flat"	{prompt="Root filename for aperature reference spectrum"}
string	apdir="tresdata$" {prompt="Directory for reference spectrum"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	refroot="comp"	{prompt="Root filename for wavelength reference spectrum"}
string	refdir="tresdata$" {prompt="Directory for reference spectrum"}
string	refmode="interp" {prompt="Method (closest, interp, fit, reference)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *procfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist, tref1, tref2, compdb
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning, nd, nf, temparg, apd, apr
bool	vb, dt, sv, fp, fx, fb, flat, disp, recent, svr, sum, sumonly, mean
int	nfiles
real	gc, tshift1, tshift2

	vb = verbose
	dt = deltemp
	sv = saveold
	svr = saveraw
	fx = fixpix
	fp = fixpreamp
	fb = fixbias
	flat = flatten
	recent = recenter
	disp = disperse
	gc = gaincorr
	nf = normfile
	nd = normdir
	apr = aproot
	apd = apdir
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPROC: Process TRES image files to extraction")
	    print ("Usage: tproc im1,im2,...,imn")
	    print ("Usage: tproc *.fits")
	    print ("Usage: tproc @image_list_file")
	    return
	    }

	sum = no
	mean = no
	sumonly = no
	if (sumspec == "yes") {
	    sum = yes
	    }
	else if (sumspec == "only") {
	    sumonly = yes
	    sum = yes
	    }
	else if (sumspec == "mean") {
	    mean = yes
	    }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempproc") // ".list"
	files (ims, >templist)

# Read through input files
	procfiles = templist
	nfiles = 0
        while (fscan (procfiles,imin) != EOF) {

# Process files as images
	    tproc1 (imin,fixpreamp=fp,fixbias=fb,fixpix=fx,deltemp=dt,saveraw=svr,saveold=sv,verbose=vb)

# Correct gain differences between amplifiers
	    tgain1 (imin,g2=gc,deltemp=dt,saveold=sv,verbose=vb)

# Merge amplifiers into a single image
	    tampmerge (imin,saveold=sv,deltemp=dt,verbose=vb)

# Flatfield correct spectra
	    if (flat)
		tflat1 (imin,normfile=nf,normdir=nd,saveold=sv,verbose=vb)
	    nfiles = nfiles + 1
	    }

# Remove cosmic rays, if requested
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		tcosmic (temparg, savebp=sv, saveold=sv, deltemp=dt, verbose=vb)
		}
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }

# Compute sum of images if requested, and add to extraction list
	if (sum) {
	    temparg = "@" // templist
	    trssum (temparg, outroot="")
	    if (sumonly) {
		print (trssum.outfile, >templist)
		}
	    else {
		print (trssum.outfile, >>templist)
		}
	    }

# Compute mean of images if requested, and add to extraction list
	if (mean) {
	    temparg = "@" // templist
	    trsavg (temparg, outroot="")
	    print (trsavg.outfile, >>templist)
	    }

# Extract 1D x orders spectrum and add dispersion, if requested
	if (extract) {
	    procfiles = templist
	    tshift1 = 0.0
	    tshift2 = 0.0
            while (fscan (procfiles,imin) != EOF) {
		textract1 (imin, aproot=apr, apdir=apd, recenter=recent, verbose=vb)
		if (disp) {
		    tfib1 (imin)
		    if (textract1.spec3 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec3, "FIB3")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec3, interactive=no, disperse=disp, saveold=sv, deltemp=dt, verbose=vb)
			    tshift1 = tcal1.shift
			    tshift2 = tcal1.shift
			    print ("TPROC: simultaneous ThAr not working yet")
			    tfib1 (textract1.spec3)
			    compdb = "comp" // tfib1.apfib // ".shift"
			    if (access (compdb))
				print (textract1.spec3,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >>compdb)
			    else
				print (textract1.spec3,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >compdb)
			    }
			else 
			    tdisp1 (textract1.spec3, saveold=sv, verbose=vb)
			}

# Compute pixel shifts for apertures 1 and 2 from existing ThAr spectra
		    else if (tfib1.object != "COMP") {
			tharshift1 (textract1.spec1)
			tref1 = tharshift.refpath
			tshift1 = tharshift1.shift
			tharshift1 (textract1.spec2)
			tref2 = tharshift.refpath
			tshift2 = tharshift1.shift
			}

		    if (textract1.spec1 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec1, "FIB1")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec1, interactive=no, disperse=disp, saveold=sv, deltemp=dt, verbose=vb)
			    tshift1 = tcal1.shift
			    tfib1 (textract1.spec1)
			    compdb = "comp" // tfib1.apfib // ".shift"
			    if (access (compdb))
				print (textract1.spec1,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >>compdb)
			    else
				print (textract1.spec1,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >compdb)
			    }
			else if (textract1.spec3 != "")
			    tdisp1 (textract1.spec1, shift=tshift1, refpath=tref1, saveold=sv, verbose=vb)
			else
			    tdisp1 (textract1.spec1, refpath="today", saveold=sv, verbose=vb)
			}
		    if (textract1.spec2 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec2, "FIB2")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec2, interactive=no, disperse=disp, saveold=sv, verbose=vb)
			    tshift2 = tcal1.shift
			    tfib1 (textract1.spec2)
			    compdb = "comp" // tfib1.apfib // ".shift"
			    if (access (compdb))
				print (textract1.spec2,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >>compdb)
			    else
				print (textract1.spec2,"	", tcal1.imref, "	", tfib1.mjdmid,"	",tcal1.shift, >compdb)
			    }
			else if (textract1.spec3 != "")
			    tdisp1 (textract1.spec2, shift=tshift2, refpath=tref2, saveold=sv, verbose=vb)
			else
			    tdisp1 (textract1.spec2, refpath="today", saveold=sv, verbose=vb)
			}
		    }
		}
	    }

	if (dt)
	    delete (templist)
	procfiles = ""
end

# Jan 09 2008	New program to process list of TRES images
# Jan 17 2008	Update normfile and normdir prompts
# Jan 24 2008	Add optional bad pixel file correction with fixpix parameter
# Jan 28 2008	Add optional cosmic ray removal from list of images with TCOSMIC
# Jan 31 2008	Add optional spectrum extraction from list of images
# Feb  8 2008	Add saveraw to save raw images but not others
# Feb  8 2008	Add optional wavelength dispersion using TDISP
# Feb 11 2008	Change temporary list file from temproclist* to tempproc*list
# Feb 12 2008	Add recenter parameter for extraction
# Feb 25 2008	Use TEXTRACT1 to process images one at a time, not TEXTRACT
# Feb 25 2008	Use TPROC1 to process images one at a time, not TPREPROC
# Feb 25 2008	Call TPREAMP1 and TGAIN1 instead of using TPREPROC
# Feb 25 2008	Change name of TDISP to TDISP1 for consistency
# Mar 18 2008	Drop deltemp from arguments to TFLAT1
# Apr 10 2008	Drop deltemp from arguments to TDISP1
# Mar 18 2008	Add TAMPMERGE
# Mar 19 2008	Run TCAL1 on COMP spectra instead of TDISP1
# Mar 26 2008	Pass saveraw to TPROC1
# Mar 31 2008	Set dispersion pixel shift from most recent calibration lamp
# Apr  2 2008	Add deltemp, saveold, and verbose arguments to tcosmic and tcal1
# Apr  2 2008	Add deltemp and verbose arguments to textract1 and tdisp1
# Apr  4 2008	Add sumspec parameter to process summed spectra
# Apr 30 2008	Move optional preamp fix to TPROC1 task
# May 22 2008	Drop unused refshift parameter
# May 23 2008	Set dispersion reference spectrum using THARSHIFT1 task
# Jul 17 2008	Save reference, time and shift of ThAr spectra in database files
# Jul 18 2008	Add refmode parameter; initialize trssum.outroot=""
# Jul 31 2008	Add mean option to sumspec parameter
# Aug 15 2008	Pass disperse parameter to TCAL1
# Sep 19 2008	Add medshift parameter to allow option of tcal1 median not mean
