# File tres/Scripts/trsgroup.cl
# September 9, 2008
# By Doug Mink

# TRSPROC: Process TRES data from lists of files made by TRSGROUP

procedure trsproc (sproc)

string	sproc="bdfto"	{prompt="Process b=ias d=ark f=lat c|t=ThAr o=bject spectra"}

string	datedir=""	{prompt="Date for images"}
file	instdir=""	{prompt="Directory containing date directories"}
file	imagedir=""	{prompt="Directory containing images (overrides above)"}
file	datafile="data.db"	{prompt="File for observation database"}
file	sortfile="datasort.db"	{prompt="File for sorted observation database"}
file	biaslist="bias.list"	{prompt="File listing groups of bias images"}
file	darklist="dark.list"	{prompt="File listing groups of dark images"}
file	flatlist="flat.list"	{prompt="File listing groups of flat fields"}
file	complist="comp.list"	{prompt="File listing groups of ThAr exposures"}
file	objlist="obj.list"	{prompt="File listing groups of observations"}
bool	verbose=yes	{prompt="Summarize results (yes or no)"}
bool	debug=no	{prompt="Print file sorting instead of writing files (yes or no)"}

struct *proclist

begin
    file image, listfile
    string filelist
    int ngroups, nbias, ndark, nobj, ncomp, nflat

# Initialize TRSGROUP
    trsgroup.images = "1-300"
    trsgroup.datedir = datedir
    trsgroup.instdir = instdir
    trsgroup.imagedir = imagedir
    trsgroup.datafile = datafile
    trsgroup.sortfile = sortfile
    trsgroup.verbose = verbose
    trsgroup.debug = debug

    nflat = 0
    nbias = 0
    ndark = 0
    ncomp = 0
    nobj = 0

# Process flat field files from lists
    smatch (sproc, "f")
    if (smatch.found) {
	if (!access (flatlist))
	    trsgroup (image)
	proclist = flatlist
    	while (fscan (proclist,listfile) != EOF) {
	    print ("TRSPROC: Processing files ",listfile," from ",flatlist)
	    filelist = "@" // listfile
	    ftres (filelist)
	    nflat = nflat + 1
	    }
	}

# Process bias files from list
    smatch (sproc, "b")
    if (smatch.found) {
	if (!access (biaslist))
	    trsgroup (image)
	proclist = biaslist
	while (fscan (proclist,listfile) != EOF) {
	    print ("TRSPROC: Processing files ",listfile," from ", biaslist)
	    filelist = "@" // listfile
	    ftres (filelist)
	    nbias = nbias + 1
	    }
	}

# Process dark files from lists
    smatch (sproc, "d")
    if (smatch.found) {
	if (!access (darklist))
	    trsgroup (image)
	proclist = darklist
	while (fscan (proclist,listfile) != EOF) {
	    print ("TRSPROC: Processing files ",listfile," from ", darklist)
	    filelist = "@" // listfile
	    ftres (filelist)
	    ndark = ndark + 1
	    }
	}

# Process Thorium Argon comparison lamp files from lists
    smatch (sproc, "t")
    if (!smatch.found)
	smatch (sproc, "c")
    if (smatch.found) {
	if (!access (complist))
	    trsgroup (image)
	proclist = complist
	while (fscan (proclist,listfile) != EOF) {
	    print ("TRSPROC: Processing files ",listfile," from ", complist)
	    filelist = "@" // listfile
	    ttres (filelist)
	    ncomp = ncomp + 1
	    }
	}

# Process object and sky files from lists
    smatch (sproc, "o")
    if (smatch.found) {
	if (!access (objlist))
	    trsgroup (image)
	proclist = objlist
	while (fscan (proclist,listfile) != EOF) {
	    print ("TRSPROC: Processing files ",listfile," from ", objlist)
	    filelist = "@" // listfile
	    ctres (filelist)
	    nobj = nobj + 1
	    }
	}
    
# Summarize the sorting process
    ngroups = nobj + nbias + ndark + nflat + ncomp
    if (verbose)
	print ("TRSPROC: ", ngroups, " groups of spectra processed")

    proclist = ""
end

# Jul 31 2008	New task
# Aug 14 2008	Use file names from trsgroup; set type(s) to process from stype
# Aug 15 2008	Move filenames back from trsgroup; separate dark and bias
# Sep  9 2008	Fix declarations
