# File tres/Scripts/tvxcsao.cl
# June 2, 2008
# By Doug Mink, SAO Telescope Data Center

# TVXCSAO - Cross-correlate TRES spectra in velocity space

procedure tvxcsao (images)

string  images=""       {prompt="Images to process"}

file	imdir=""	{prompt="Directory containing images to analyze"}
string	template=""	{prompt="Template (null is reference template)"}
string	tempdir=""	{prompt="Template directory"}
string	orders="1-51"	{prompt="Range of orders to cross-correlate (TRES=1-51)"}
int	pkmode=1	{prompt="Peak-fitting mode (1-parabola 2-quartic 3-cosx/1+x^2)"}
real	pkfrac=0.5	{prompt="Fraction of peak to fit (0.95 for mode 3 0.5 for mode 1"}
int	nbins=65536	{prompt="Number of bins for transform (power of two)"}
string	zeropad="no"	{prompt="Pad spectrum transform with equal number of zeroes (yes no tempfile)"}
bool	hifilter=no	{prompt="High-pass filter transforms (yes or no)"}
bool	displot=no	{prompt="Display results graphically (yes or no)"}
file	logfile=""	{prompt="File for per-spectrum velocity results"}
bool	curmode=no	{prompt="Stop after each order (yes or no)"}

struct *calfiles

begin
    file imin, templist, imout, xlog
    int nb
    string ims


    nb = nbins

    ims = images
    if (logfile == "") {
	xlog = "tvxcsao_mean.log"
	}
    else {
	xlog = logfile
	}

    if (!deftask ("pxcsao")) {
	rvsao (motd=no)
	}

    pxcsao.specnum = orders
    pxcsao.specband = 0
    pxcsao.specdir = imdir
    pxcsao.correlate = "velocity"
    pxcsao.tempnum = "1-51"
    pxcsao.tempband = 1
    pxcsao.echelle = yes
    pxcsao.st_lambda = INDEF
    pxcsao.end_lambda = INDEF
    pxcsao.obj_plot = no
    pxcsao.xcor_plot = no
    pxcsao.xcor_file = no
    pxcsao.fixbad = no
    pxcsao.badlines = "tharbad.dat"
    pxcsao.s_emchop = "no"
    pxcsao.t_emchop = "no"
    pxcsao.s_abs_reject = 100.
    pxcsao.s_em_reject = 2.
    pxcsao.t_abs_reject = 0.
    pxcsao.t_em_reject = 0.
    pxcsao.bell_window = 0.05
    pxcsao.renormalize = no
    pxcsao.ncols = nb
    pxcsao.interp_mode = "sums"
    pxcsao.zeropad = zeropad
    pxcsao.low_bin = 5
    pxcsao.top_low = 20
    if (!hifilter) {
	topnrun = pxcsao.top_nrun
	nrun = pxcsao.nrun
	pxcsao.top_nrun = nb
	pxcsao.nrun = nb
	}
    pxcsao.vel_init = "zero"
    pxcsao.czguess = 0.
    pxcsao.nzpass = 0
    pxcsao.tshift = 0.
    pxcsao.svel_corr = "file"
    pxcsao.tvel_corr = "file"
    pxcsao.pkmode = pkmode
    pxcsao.pkfrac = pkfrac
    pxcsao.pksrch = 25
    pxcsao.minvel = -50.
    pxcsao.maxvel = 50.
    pxcsao.report_mode = 18
    pxcsao.logfiles = "STDOUT,tvxcsao.log"
    pxcsao.save_vel = no
    pxcsao.rvcheck = no
    pxcsao.archive = no
    pxcsao.nsmooth = 0
    pxcsao.cvel = INDEF
    pxcsao.dvel = INDEF
    pxcsao.ablines = ""
    pxcsao.emlines = "thar.dat"
    pxcsao.linedir = "tres$lib/"
    pxcsao.dispmode = 1
    pxcsao.displot = displot
    pxcsao.device = "stdgraph"
    pxcsao.vel_plot = "correlation"
    pxcsao.curmode = curmode
    pxcsao.hardcopy = no
    pxcsao.plotter = "stdplot"
    pxcsao.besttemp = ""
    pxcsao.velocity = 0.0
    pxcsao.velerr = 0.00
    pxcsao.r = 0.0
    pxcsao.temp_plot = no
    pxcsao.contsub_plot = no
    pxcsao.apodize_plot = no
    pxcsao.fft_plot = no
    pxcsao.tfft_plot = no
    pxcsao.uxcor_plot = no
    pxcsao.nsum = 1
    pxcsao.debug = no
    pxcsao.cursor = ""

#  Make temporary list of input files
    templist = mktemp ("tempcomp") // ".list"
    files (ims, >templist)

    calfiles = templist
    while (fscan (calfiles, imin) != EOF) {

	if (template == "") {
	    tidref (imin)
	    pxcsao.templates = tidref.reffile
	    pxcsao.tempdir = tidref.refdir
	    }
	else {
	    pxcsao.templates = template
	    pxcsao.tempdir = tempdir
	    }
	pxcsao (imin)
	print ("TVXCSAO: ",imin,", vel=",pxcsao.velocity," +-",pxcsao.velerr)
	print (imin," ",pxcsao.velocity," ",pxcsao.velerr, >> xlog)
	}

    if (!hifilter) {
	pxcsao.top_nrun = topnrun
	pxcsao.nrun = nrun
	}
    calfiles = ""
    delete (templist)
end
# May 23 2008	New task based on TPXCSAO
# May 29 2008	Add new parameters imdir, nbins, pkmode, displot, curmode
# June 2 2008	Add new parameters zeropad, hifilter
