# File tres/Src/t_tfib1.x
# By Doug Mink
# August 14, 2008

# TFIB1 -- Figure out and optionally set APFIB, FIB1, FIB2, and FIB3
#            keywords in a TRES data file using the values of other keywords

include <error.h>
include <imhdr.h>
include <imio.h>
include <ctype.h>

define	LEN_USER_AREA	100000

procedure t_tfib1 ()

char spectrum[SZ_FNAME]	# File to process
char specdir[SZ_PATHNAME]	# Input directory for data files
bool addfib		# Add FIBi keywords to spectra (yes or no)

char imagetype[SZ_LINE]	# IRAF image type
char dateobs[SZ_LINE]	# Image date and time of observation
char object[SZ_LINE]	# Object name
char peri[8]		# Periscope IN (Fibers 1 and 2) or OUT (fiber 3)
char bin1[8]		# Binning (1 or 2)
char fibsize[8]		# Fiber size (1.3 2.3 2.8)"}
char osswitch[8]	# NO: 1=object 2=sksky  YES: 1=sky 2=object
char exptime[8]		# Exposure time in seconds
char comptime[8]	# Comparison lamp flash (peri=OUT) time in sec
char fibkey[8]		# Fiber usage, one character per fiber (OSNCF)
char fib1[8]		# Fiber 1 (usually object) content (returned)
char fib2[8]		# Fiber 2 (usually sky) content (returned)
char fib3[8]		# Fiber 3 (usually comp) content (returned)
char binning[8]		# TRES binning (b if 2, null if 1) (returned)
char fiber[8]		# TRES fiber(s) (1 2 3 12 123) (returned)
char apfib[8]		# TRES aperture size (s m l) binning (b null) fiber (1 2 3)
char apsize[8]		# TRES aperture size (s=1.8, m=2.3, l=2.9) (returned)
bool vb			# Log progress (yes or no)
bool db			# Print debugging information (yes or no)

bool fibinhead
int ldir, mode, ip
char imf[SZ_PATHNAME]
char imf0[SZ_PATHNAME]
char imf1[SZ_PATHNAME]
char indir[SZ_PATHNAME]
char ccdsum[8]
pointer im, im1
double mjd, mjdmid, et

bool clgetb()
int strlen(), strcmp()
pointer immap()
errchk  immap()

begin

	vb = clgetb ("verbose")
	db = clgetb ("debug")
	call clgstr ("spectrum", spectrum, SZ_FNAME)
	call clgstr ("specdir", specdir, SZ_PATHNAME)
	call strcpy (specdir, indir, SZ_PATHNAME)
	addfib = clgetb ("addfib")
	ldir = strlen (indir)
	if (ldir < 1)
	    call strcpy (spectrum, imf, SZ_PATHNAME)
	else {
	    if (indir[ldir] != '/') {
		indir[ldir+1] = '/'
		indir[ldir+2] = EOS
		}
	    call strcpy (indir, imf, SZ_PATHNAME)
	    call strcat (spectrum, imf, SZ_PATHNAME)
	    }
	call strcpy (imf, imf0, SZ_PATHNAME)
	call strcat ("[0]", imf0, SZ_PATHNAME)
	if (db) {
	    call printf ("TFIB1: reading %s\n")
		call pargstr (imf0)
	    call flush (STDOUT)
	    }

# Open image file to read header
	if (addfib)
	    mode = READ_WRITE
	else
	    mode = READ_ONLY
	iferr (im = immap (imf0, mode, 0)) {
            if (im != NULL)
		call imunmap (im)
	    if (db) {
		call printf ("TFIB1: image file %s read error\n")
		    call pargstr (imf0)
		call flush (STDOUT)
		}
	    return
	    }
	else if (db) {
	    call printf ("TFIB1: image file %s read\n")
		call pargstr (imf0)
	    call flush (STDOUT)
	    }

# Read important keyword values from this file
	call strcpy ("", dateobs, SZ_LINE)
	call imgspar (im, "DATE-OBS", dateobs, SZ_LINE)
	call strcpy ("", imagetype, SZ_LINE)
	call imgspar (im, "IMAGETYP", imagetype, SZ_LINE)
	call strcpy ("", object, SZ_LINE)
	call imgspar (im, "i_title", object, SZ_LINE)
	call strcpy ("IN", peri, 8)
	call imgspar (im, "PERI", peri, 8)
	call strcpy ("1.4", fibsize, 8)
	call imgspar (im, "FIBSIZE", fibsize, 8)
	call strcpy ("0", ccdsum, 8)
	call strcpy ("NO", osswitch, 8)
	call strcpy ("0.0", exptime, 8)
	call strcpy ("0.0", comptime, 8)
	call imgspar (im, "CCDSUM", ccdsum, 8)
	call imgdpar (im, "MJD", mjd)
	ip = 1
	call ctod (exptime, ip, et)
	mjdmid = mjd + ((0.5d0 * et) / 8.64d4)
	if (ccdsum[1] == '0') {
	    call strcpy (imf, imf1, SZ_FNAME)
	    call strcat ("[1]", imf1, SZ_FNAME)
	    iferr (im1 = immap (imf1, READ_ONLY, 0)) {
        	if (im1 != NULL)
		    call imunmap (im1)
		if (db) {
		    call printf ("TFIB1: image file %s read error\n")
			call pargstr (imf1)
		    call flush (STDOUT)
		    }
		return
		}
	    else if (db) {
		call printf ("TFIB1: image file %s read\n")
		    call pargstr (imf1)
		call flush (STDOUT)
		}
	    call imgspar (im1, "CCDSUM", ccdsum, 8)
	    call imgspar (im1, "OSSWITCH", osswitch, 8)
	    call imgspar (im1, "EXPTIME", exptime, 8)
	    call imgspar (im1, "COMPTIME", comptime, 8)
	    call imunmap (im1)
	    }
	else {
	    call imgspar (im, "OSSWITCH", osswitch, 8)
	    call imgspar (im, "EXPTIME", exptime, 8)
	    call imgspar (im, "COMPTIME", comptime, 8)
	    }

# Read pre-existing FIBi keywords, but ignore them if blank
	call strcpy ("___", fib1, 8)
	call imgspar (im, "FIB1", fib1, 8)
	if (strcmp (fib1, "        ") == 0)
	    call strcpy ("___", fib1, 8)
	if (strlen (fib1) == 0)
	    call strcpy ("___", fib1, 8)
	call strcpy ("___", fib2, 8)
	call imgspar (im, "FIB2", fib2, 8)
	if (strcmp (fib2, "        ") == 0)
	    call strcpy ("___", fib2, 8)
	if (strlen (fib2) == 0)
	    call strcpy ("___", fib2, 8)
	call strcpy ("___", fib3, 8)
	call imgspar (im, "FIB3", fib3, 8)
	if (strcmp (fib3, "        ") == 0)
	    call strcpy ("___", fib3, 8)
	if (strlen (fib3) == 0)
	    call strcpy ("___", fib3, 8)

# Read FIBKEY keyword
	call strcpy ("___", fibkey, 8)
	call imgspar (im, "FIBKEY", fibkey, 8)

	if (db) {
	    call printf ("TFIB1: filename= %s  object= %s\n")
	    	call pargstr (imf)
	    	call pargstr (object)
	    call printf ("TFIB1: peri= %s  ccdsum= %s fibkey= %s\n")
		call pargstr (peri)
		call pargstr (ccdsum)
		call pargstr (fibkey)
		call printf ("TFIB1: fibsize= %s  osswitch= %s\n")
		call pargstr (fibsize)
		call pargstr (osswitch)
	    call printf ("TFIB1: exptime= %s  comptime= %s\n")
		call pargstr (exptime)
		call pargstr (comptime)
	    }

# If FIBKEY keyword exists, set FIBi from it
	if (fibkey[1] != '_') {
	    if (fibkey[1] == 'C')
		call strcpy ("COMP", fib1, 8)
	    else if (fibkey[1] == 'F')
		call strcpy ("FLAT", fib1, 8)
	    else if (fibkey[1] == 'S')
		call strcpy ("SKY", fib1, 8)
	    else if (fibkey[1] == 'O')
		call strcpy ("OBJ", fib1, 8)
	    else
		call strcpy ("OFF", fib1, 8)
	    if (fibkey[2] == 'C')
		call strcpy ("COMP", fib2, 8)
	    else if (fibkey[2] == 'F')
		call strcpy ("FLAT", fib2, 8)
	    else if (fibkey[2] == 'S')
		call strcpy ("SKY", fib2, 8)
	    else if (fibkey[2] == 'O')
		call strcpy ("OBJ", fib2, 8)
	    else
		call strcpy ("OFF", fib2, 8)
	    if (fibkey[3] == 'C')
		call strcpy ("COMP", fib3, 8)
	    else if (fibkey[3] == 'F')
		call strcpy ("FLAT", fib3, 8)
	    else if (fibkey[3] == 'S')
		call strcpy ("SKY", fib3, 8)
	    else if (fibkey[3] == 'O')
		call strcpy ("OBJ", fib3, 8)
	    else
		call strcpy ("OFF", fib3, 8)
	    if (db) {
		call printf ("From FIBKEY= %s, setting FIB1= %s  FIB2= %s  FIB3= %s\n")
		    call pargstr (fibkey)
		    call pargstr (fib1)
		    call pargstr (fib2)
		    call pargstr (fib3)
		}
	    }

# Set FIBi values if not in header already
	fibinhead = TRUE
	if (strcmp(fib1,"___")==0 || strcmp(fib2,"___")==0 || strcmp(fib3,"___")==0){
	    fibinhead = FALSE

	    if (strcmp (imagetype, "FLAT") == 0) {
		if (strcmp (peri, "IN") == 0) {
		    call strcpy ("FLAT", fib1, 8)
		    call strcpy ("FLAT", fib2, 8)
		    call strcpy ("OFF", fib3, 8)
		    call strcpy ("FFN", fibkey, 8)
		    }
		else {
		    call strcpy ("OFF", fib1, 8)
		    call strcpy ("OFF", fib2, 8)
		    call strcpy ("FLAT", fib3, 8)
		    call strcpy ("NNF", fibkey, 8)
		    }
		}
	    else if (strcmp (imagetype, "COMP") == 0) {
		if (strcmp (peri, "IN") == 0) {
		    call strcpy ("COMP", fib1, 8)
		    call strcpy ("COMP", fib2, 8)
		    call strcpy ("OFF", fib3, 8)
		    call strcpy ("CCN", fibkey, 8)
		    }
		else {
		    call strcpy ("OFF", fib1, 8)
		    call strcpy ("OFF", fib2, 8)
		    call strcpy ("COMP", fib3, 8)
		    call strcpy ("NNC", fibkey, 8)
		    }
		}
	    else if (strcmp (imagetype, "OBJECT") == 0) {
		if (strcmp (object, "COMP") == 0) {
		    if (strcmp (peri, "IN") == 0) {
			call strcpy ("COMP", fib1, 8)
			call strcpy ("COMP", fib2, 8)
			call strcpy ("OFF", fib3, 8)
			call strcpy ("CCN", fibkey, 8)
			}
		    else {
			call strcpy ("OFF", fib1, 8)
			call strcpy ("OFF", fib2, 8)
			call strcpy ("COMP", fib3, 8)
			call strcpy ("NNC", fibkey, 8)
			}
		    }
		else if (strcmp (comptime, "0.0") != 0) {
		    if (strcmp (peri, "IN") == 0) {
			if (strcmp (osswitch, "YES") == 0) {
			    call strcpy ("SKY", fib1, 8)
			    call strcpy ("OBJ", fib2, 8)
			    call strcpy ("COMP", fib3, 8)
			    call strcpy ("SOC", fibkey, 8)
			    }
			else {
			    call strcpy ("OBJ", fib1, 8)
			    call strcpy ("SKY", fib2, 8)
			    call strcpy ("COMP", fib3, 8)
			    call strcpy ("OSC", fibkey, 8)
			    }
			}
		    else {
			call strcpy ("OFF", fib1, 8)
			call strcpy ("OFF", fib2, 8)
			call strcpy ("COMP", fib3, 8)
			call strcpy ("NNC", fibkey, 8)
			}
		    }
		else {
		    if (strcmp (peri, "OUT") == 0) {
			if (strcmp (osswitch, "YES") == 0) {
			    call strcpy ("SKY", fib1, 8)
			    call strcpy ("OBJ", fib2, 8)
			    call strcpy ("OFF", fib3, 8)
			    call strcpy ("SON", fibkey, 8)
			    }
			else {
			    call strcpy ("OBJ", fib1, 8)
			    call strcpy ("SKY", fib2, 8)
			    call strcpy ("OFF", fib3, 8)
			    call strcpy ("OSN", fibkey, 8)
			    }
			}
		    else {
			if (strcmp (osswitch, "YES") == 0) {
			    call strcpy ("SKY", fib1, 8)
			    call strcpy ("OBJ", fib2, 8)
			    call strcpy ("COMP", fib3, 8)
			    call strcpy ("SOC", fibkey, 8)
			    }
			else {
			    call strcpy ("OBJ", fib1, 8)
			    call strcpy ("SKY", fib2, 8)
			    call strcpy ("COMP", fib3, 8)
			    call strcpy ("OSC", fibkey, 8)
			    }
			}
		    }
		}
	    else if (strcmp (imagetype, "DARK") == 0) {
		call strcpy ("OFF", fib1, 8)
		call strcpy ("OFF", fib2, 8)
		call strcpy ("OFF", fib3, 8)
		call strcpy ("NNN", fibkey, 8)
		}
	    else if (strcmp (imagetype, "ZERO") == 0) {
		call strcpy ("OFF", fib1, 8)
		call strcpy ("OFF", fib2, 8)
		call strcpy ("OFF", fib3, 8)
		call strcpy ("NNN", fibkey, 8)
		}
	    if (db) {
		call printf ("Setting FIBKEY= %s  FIB1= %s  FIB2= %s  FIB3= %s\n")
		    call pargstr (fibkey)
		    call pargstr (fib1)
		    call pargstr (fib2)
		    call pargstr (fib3)
		}
	    }

	if (fibkey[1] == 'N' && fibkey[2] == 'N' && fibkey[3] == 'N')
	    call strcpy ("", fiber, 8)
	else if (fibkey[1] == 'N' && fibkey[2] == 'N')
	    call strcpy ("3", fiber, 8)
	else if (fibkey[2] == 'N' && fibkey[3] == 'N')
	    call strcpy ("1", fiber, 8)
	else if (fibkey[1] == 'N' && fibkey[3] == 'N')
	    call strcpy ("2", fiber, 8)
	else if (fibkey[3] == 'N')
	    call strcpy ("12", fiber, 8)
	else
	    call strcpy ("123", fiber, 8)

	bin1[1] = ccdsum[1]
	bin1[2] = EOS
	if (bin1[1] == '2')
	    call strcpy ("b", binning, 8)
	else
	    binning[1] = EOS
	if (strcmp (fibsize, "1.4") == 0)
	    call strcpy ("s", apsize, 8)
	else if (strcmp (fibsize, "2.3") == 0)
	    call strcpy ("m", apsize, 8)
	else
	    call strcpy ("l", apsize, 8)

# Set apfib used for ThAr and FLAT
	call strcpy (apsize, apfib, 8)
	call strcat (binning, apfib, 8)
	call strcat (fiber, apfib, 8)

# Add FIBi keywords to input file if requested
	IM_UPDATE(im) = NO
	if (addfib && !fibinhead) {
	    if (db) {
		call printf ("Setting FIB1= %s  FIB2= %s  FIB3= %s APFIB=%s\n")
		   call pargstr (fib1)
		   call pargstr (fib2)
		   call pargstr (fib3)
		   call pargstr (apfib)
		}
	    IM_UPDATE(im) = YES
	    call imastr (im, "FIBKEY", fibkey)
	    call imastr (im, "FIB1", fib1)
	    call imastr (im, "FIB2", fib2)
	    call imastr (im, "FIB3", fib3)
	    call imastr (im, "APFIB", apfib)
	    }

# Close image
	call imunmap (im)

	if (vb) {
	    call printf ("%s	%s	%s	%s	%.5f	%s	%3s	%s	%s\n")
		call pargstr (imf)
		call pargstr (imagetype)
		call pargstr (object)
		call pargstr (dateobs)
		call pargd (mjdmid)
		call pargstr (apfib)
		call pargstr (fibkey)
		call pargstr (exptime)
		call pargstr (comptime)
	    }

# Write spectrum parameters to task parameter file
	call clpstr ("spectrum", spectrum)
	call clpstr ("specdir", specdir)
	call clpstr ("imagetype", imagetype)
	call clpstr ("dateobs", dateobs)
	call clpstr ("object", object)
	call clpstr ("apfib", apfib)
	call clpstr ("peri", peri)
	call clpstr ("bin1", bin1)
	call clpstr ("fibsize", fibsize)
	call clpstr ("osswitch", osswitch)
	call clpstr ("exptime", exptime)
	call clpstr ("comptime", comptime)
	call clpstr ("fibkey", fibkey)
	call clpstr ("fiber", fiber)
	call clpstr ("fib1", fib1)
	call clpstr ("fib2", fib2)
	call clpstr ("fib3", fib3)
	call clpstr ("binning", binning)
	call clpstr ("apsize", apsize)
	call clputd ("mjd", mjd)
	call clputd ("mjdmid", mjdmid)


end
# Oct  3 2007	New program
# Oct  5 2007	Fix FIBi writing; replace input spaces in strings with _
# Oct 11 2007	Ignore blank FIBi keywords
# Oct 15 2007	Clean up declarations
# Oct 26 2007	Use binning and times from ext. 1 if not in primary header
# Nov 30 2007	Add IMTYPE=COMP for calibration lamp spectra (was OBJECT)
# Dec  4 2007	Set FIB1, FIB2, and FIB3 from 3-character FIBKEY
# Dec  5 2007	Print FIBKEY value, too
# Dec 11 2007	Always read FIBKEY, and fix bug to write parameter fibkey

# Jan 10 2008	Change name from TRESFIB to TRSFIB
# Jan 28 2008	Add fiber string
# Feb 13 2008	Set fibi parameters and keywords from FIBKEY, if it is present
# Feb 15 2008	Add date and time of observation from FITS header
# May 22 2008	Add mjd, Modified Julian Day
# May 23 2008	Add apfib for ThAr and FLAT identification
# May 30 2008	Add mjdmid midtime for exposure
# Jun  3 2008	Write APFIB to header if writing FIBi
# Jun  6 2008	Change name to TRSFIB1 because it works on one file at a time
# Aug  7 2008	Call ctod() to convert exptime string to double
# Aug 11 2008	Update verbose output to include apfib and mjdmid
# Aug 11 2008	Change name from TRSFIB1 to TFIB1
# Aug 14 2008	Redimension apsize to 8 characters from 1 character
